#!/bin/bash

help()
{
	cat << EOT

ROCKET - ROCK nEtwork packeTmanager ... or something alike ... ;-)

The GEM Pools (install sources) are configured in /etc/rocket.conf

rocket updsrc .............. update local rock source tree
rocket emerge <pkg> ........ build a package from sources

rocket create_gem <pkg> .... create GEM file from local package
rocket index ............... create the index file for a directory with GEMs

rocket update .............. update the package cache
rocket install <pkg> ....... install/update package

EOT
	exit 1
}

upd_archive() {
	echo "Reading $1/packages.db ..."
	awk '

BEGIN { chunk=0; pkg="NONE"; ver="UNKOWN"; }

$0 == "\027" { chunk++; next; }
$0 == "\004" {
	close("/var/adm/rocket/descs/" pkg);
	close("/var/adm/rocket/dependencies/" pkg);
	close("/var/adm/rocket/cksums/" pkg);
	print pkg " '"$1"'/" pkg "-" ver ".gem" \
		>> "/var/adm/rocket/locations.txt";
	chunk=0; pkg="NONE"; next;
}

chunk == 0 { pkg=$0; }
chunk == 1 { print > "/var/adm/rocket/descs/" pkg; }
chunk == 2 { print > "/var/adm/rocket/dependencies/" pkg; }
chunk == 3 { print > "/var/adm/rocket/cksums/" pkg; }

chunk == 1 && $1 == "[V]" { ver=$2; }

' < <( curl -s "$1/packages.db" | gunzip; )
}

case "$1" in

    updsrc)
	cd /usr/src/rock-src || exit 1
	./scripts/Update-Src
	./scripts/Cleanup
	;;

    emerge)
	shift; cd /usr/src/rock-src || exit 1
	if [ ! -d config/rocket ]; then
		mkdir -p config/rocket
		cp -r /etc/ROCK-CONFIG/* config/rocket/
		./scripts/Config -cfg rocket -oldconfig
	fi
	exec -a "$0" ./scripts/Emerge-Pkg -cfg rocket "$@"
	;;

    create_gem)
	shift
	for pkg; do
		v="$(grep '^Package Name and Version' /var/adm/packages/$pkg | cut -f6 -d' ')"
		mine -T /var/adm / "$pkg" "$pkg-$v.tar.bz2"
		mine -C /var/adm "$pkg-$v.tar.bz2" "$pkg" "$pkg-$v.gem"
	done
	;;

    update)
	rm -rf /var/adm/rocket
	mkdir -p /var/adm/rocket/{descs,dependencies,cksums}
	while read keyword p1 p2; do
		case "$keyword" in
		    archive)
			upd_archive "$p1"
			;;
		    *)
			echo "Unknown keyword '$keyword' in config file!" >&2
			;;
		esac
	done < <( grep '^ *[^#]' /etc/rocket.conf | tac; )
	;;

    install)
	shift
	for x; do
		url="$( grep "^$x " /var/adm/rocket/locations.txt | cut -f2 -d' ' | tail -1; )"
		if [ -z "$url" ]; then
			echo "Package $x not found - maybe you need to add the archive"
			echo "to /etc/rocket.conf and run 'rocket update' first."
		else
			echo "+ mine -i '$url'"; mine -i "$url"
		fi
	done
	;;

    index)
	echo_error() { :; }
	ROCKCFG_CREATE_GEM=1
	ROCKCFG_PKGFILE_VER=1
	. /usr/src/rock-src/scripts/functions
	create_package_db /var/adm . packages.db
	;;

    *)
	help
	;;
esac

